/**
 *
 */


Ext.define('FP.controller.FrontPanelControl', {
    extend: 'Ext.app.Controller',
    requires: ['Ext.Ajax', 'FP.utils.CmdInterface'],
    /**
     * @cfg {Object} keyInfo
     * A map of keys to their relative presscode and data location in the binary blob
     */
    keyInfo: {
        'key_up': 
            {
                pressCode: '!D'
            },
        'key_down': 
            {
                pressCode: '!U'
            },
        'key_upfast': 
            {
                pressCode: '!X'
            },
        'key_downfast': 
            {
                pressCode: '!x'
            },
        'key_1': 
            {
                pressCode: '!1',
                dataLocation: {
                    offset: 4,
                    shift: 6
                }
            },
        'key_2': 
            {
                pressCode: '!2',
                dataLocation: {
                    offset: 4,
                    shift: 4
                }
            },
        'key_3': 
            {
                pressCode: '!3',
                dataLocation: {
                    offset: 4,
                    shift: 2
                }
            },
        'key_4': 
            {
                pressCode: '!4',
                dataLocation: {
                    offset: 4,
                    shift: 0
                }
            },
        'key_5': 
            {
                pressCode: '!5',
                dataLocation: {
                    offset: 3,
                    shift: 2
                }
            },
        'key_6':             
            {
                pressCode: '!6',
                dataLocation: {
                    offset: 3,
                    shift: 4
                }
            },
        'key_logo': 
            {
                pressCode: '!7',
                dataLocation: {
                    offset: 2,
                    shift: 6
                }
            },
        'key_sel':             
            {
                pressCode: '!S'
            },
        'key_esc':             
            {
                pressCode: '!E'
            },
        'key_frz': 
            {
                pressCode: '!I',
                dataLocation: {
                    offset: 3,
                    shift: 6
                }
            },
        'key_setup': 
            {
                pressCode: '!F',
                dataLocation: {
                    offset: 3,
                    shift: 6
                }
            },
        'key_blk': 
            {
                pressCode: '!8',
                dataLocation: {
                    offset: 3,
                    shift: 0
                }
            },
        'key_tp': 
            {
                pressCode: '!B'
                
            },
        'key_panzoom': 
            {
                pressCode: '!K'
                
            }
        
    },    
    init: function() {
        "use strict";
        var me = this;
        this.cmdIface = Ext.create('FP.utils.CmdInterface');
        this.sendKeyCmd('!R');
        window.setInterval(function() {
            var nowMillis = new Date().getTime();
            if( (nowMillis - me.lastTimeSent) > 2000) {
                //delay if user input
                me.sendKeyCmd('!R');
            }
        }, 4000);
    },
    /**
     * sends command to unit on key pressed. The deriving controller should 
     * register this method for its key events
     * @param {String} name Name of the key pressed
     */
    onViewKeyPressed: function(name) {
        "use strict";
        var keyCmd = this.keyInfo[name];
        if(keyCmd !== undefined && keyCmd.pressCode !== undefined) {
            this.sendKeyCmd(keyCmd.pressCode);
        }
    },
    sendKeyCmd: function(keyCode) {
        "use strict";
        var me = this;
        this.lastTimeSent = new Date().getTime();
        this.cmdIface.sendCmd(keyCode, function(res) {
            me.loadBlob(res);
        });
    },
    loadBlob: function(res) {
        "use strict";
        
        var lines = res.responseText.split('\n'),
            decode_byte = null,
            decoded=[],
            reduce = null,
            reduced = [],
            normalize = null,
            me = this;
        
        this.lastTimeSent = new Date().getTime();
        decode_byte = function(byteIn) {
            var ret;
            if(byteIn == 0x60) {
                ret = 0;
            }
            else {
                ret = byteIn - 0x20;
            }
            //do what the implicit byte cast would do. 
            return Math.abs(ret);
        };
        
        reduce = function() {
            var idx = 0,
                intArray = null;
            for(; idx < decoded.length; idx+=1) {
                intArray = decoded[idx];
                reduced.push.apply(reduced, intArray);
            }
            me.reduced = reduced;
            me.doDisplay(reduced);
            me.doLeds(reduced);
        };
        
        //normalize function does what a (byte) cast would do in java
        normalize = function(bitValue) {
            return bitValue % 256;
        };
        
        //uudecodeLine
        Ext.Array.forEach(lines, function(strline, idx) {
            
            var i = 0,
                j = 0,
                p = 1,
                lengthHeader = null,
                out = null;

            if(strline.length !== 0 ){

                lengthHeader = decode_byte(strline.charCodeAt(0));
                out = [];

                while(strline.charCodeAt(p) !== 0) {
                    out[j] = normalize(decode_byte(strline.charCodeAt(p)));
                    out[j+1] = normalize(decode_byte(strline.charCodeAt(p+1)));
                    out[j] = normalize( (out[j] << 2) );
                    out[j] |= normalize((out[j+1] >> 4) & 0x3);
                    out[j+1] = normalize(out[j+1] << 4);
                    out[j+2] = normalize(decode_byte(strline.charCodeAt(p+2)));
                    out[j+1] = normalize(out[j+1] | (normalize(out[j+2] >> 2) & 0x0F));
                    out[j+2] = normalize(out[j+2] << 6);
                    out[j+2] = normalize(out[j+2] | (decode_byte(strline.charCodeAt(p+3)) & 0x3F));

                    j+=3;
                    p+=4;
                    i+=3;
                    if(i >= lengthHeader) {
                        break;
                    }

                }

                decoded[idx] = out;
            }
            if(idx === lines.length - 1) {
                //when finished decoding the entire chunk, reduce it to a single array
                reduce();
            } 
        });
        
    },
    
    /*Again, this is a mirror of java code  */
    /* Function takes decoded 
     * @data String 
     *  and converts to a list of strings that can be displayed on a VFD*/
    doDisplay: function(data) {
        "use strict";
        //only parts of the data can be converted into strings directly.
        //there's more information packed in there than just ascii. 
        var converted = [],
            convert = function(begin, end, textIdx) {
            var idx = begin,
                convertedStr = '';
            for(;idx < end; begin += 1) {
                if(data[idx] == 0) {
                    convertedStr+=' ';
                } else {
                    convertedStr+=String.fromCharCode(data[idx]);
                }
                idx += 1;
            }
//            convertedStr += '\n'
            converted.push(convertedStr);
        };
        convert(16, 40, 0);
        convert(43, 67, 1);
        convert(70, 94, 2);
        convert(97, 121, 3);

        this.controllerSetDisplayData(converted);
        this.fireEvent('displaychange', converted);
    },
    
    /* 
     * Virtual
     * load the specific display view with data. 
     * */
    controllerSetDisplayData: function(converted) {
        "use strict";
        BLogger.log('not implemented ' + converted);
    },
    
    /* see java/c++ code for a true explanation
     *   each led has four possible states, 
     *      off (0)
     *      on (1) 
     *      blink-slow (2) 
     *      blink-fast (3)
     *      
     * */
    doLeds: function(data) {
        "use strict";
        var loc,
            ledState = {},
            prop;
        for(prop in this.keyInfo) {
            if(this.keyInfo.hasOwnProperty(prop)){
                loc = this.keyInfo[prop].dataLocation;
                if(loc !== undefined) {
                    ledState[prop] = (data[loc.offset] >> loc.shift) & 3; // 3 bits to mask 4 values
                }
            }
        }
        this.controllerSetLedData(ledState);
        this.fireEvent('ledchange', ledState);
    },
    
    /* 
     * Virtual
     * load the specific LED view with data. 
     * */
    controllerSetLedData: function(converted) {
        "use strict";
        BLogger.log('not implemented ');
        BLogger.log(converted);
    }
    
});
