/**
 * A class responsible for creating IFrame slave connections
 * 
 * 
 */
Ext.define('Muc.manager.IFrameManager', {
    extend: 'Ext.util.Observable',
    config: {
        store: null,
        controller: null
    },
    
    
    /**
     * @cfg {Ext.data.Store} store
     * @cfg {Ext.app.Controller} controller
     */

    constructor: function(config) {
        var me = this;
        this.iframes = {};
        this.iframeAges = {};
        this.iframeEvts = {};
        Ext.get(window).addListener('message', this.onIFrameMessage, this);
        this.initConfig(config);
        this.callParent(arguments);
        
        window.setInterval(function(){
            var ip;
            for(ip in me.iframeAges) {
                if(me.iframeAges.hasOwnProperty(ip)) {
                    me.iframeAges[ip] += 1;
                }
                if(me.iframeAges[ip] >= 6) {
                    me.controller.updateConnectionState(ip, 'Disconnected');
                }
            }
            
        }, 3000)
    },
    /**
     * @private
     * @param {Object} store adds IFrames from records of store and hooks 
     *  into add/update for future creation
     */
    applyStore: function(store) {
        var me = this;
        if(store) {
            this.populateIFramesFromStore(store);
            store.on('load', this.populateIFramesFromStore, this);
            
            store.addListener('add', function(store, records) {
                console.log("IFrameManager, ADD");
                var idx;
                for(idx = 0; idx < records.length; idx+=1) {
                    me.createIFrameFromRecord(records[idx]);
                }
            });
            store.addListener('remove', function(store, record) {
                console.log("IFrameManager, Delete");
                me.deleteIFrame(record.get('ip-address'));
            });
            store.addListener('update', function(storeIn, record, operation, modifiedFieldNames) {
                console.log("IFrameManager, OPERATION = " + operation);
                if(modifiedFieldNames && Ext.Array.indexOf(modifiedFieldNames, 'ip-address') >= 0) {
                    me.createIFrameFromRecord(record);
                }
            });
            
        }
        return store;
    },
    /**
     *Create iframe for comm to slave units
     * @param {String} ip ip address of unit. ex: '192.168.1.2'
     */
    createIFrame: function(ip) {
        var httpSrc;
        if(ip && ip !== "") {
            if(typeof this.iframes[ip] === 'undefined') {
                httpSrc = 'http://'+ip;
                console.log('creating IFrame at ' + ip);
                this.controller.updateConnectionState(ip, 'Disconnected');
                this.iframeAges[ip] = 0;
                this.iframes[ip]=Ext.create('Muc.view.IFrame', {renderTo: Ext.getBody(), renderData: {iframeSrc:httpSrc}});
            }
        }
    },
    /**
     * Delete the iframe element for given ip. 
     *@param {String} ip IP of iframe to delete
     */
    deleteIFrame: function(ip) {
        var iframe = this.iframes[ip];
        if(iframe) {
            iframe.destroy();
            delete this.iframes[ip];
            delete this.iframeAges[ip];
        }
    },
    /**
     *  Create IFrame UnitM record.
     *  @param {Ext.data.Record} record
     */
    createIFrameFromRecord: function(record) {
        this.createIFrame(record.get('ip-address'));
    },
    
    onIFrameMessage: function(msg) {
        var evt = msg.browserEvent,
            ip = evt.origin.replace('http://', ''),
            controller = this.getController();
            this.iframeEvts[ip] = evt;
            this.iframeAges[ip] = 0;
            
        if(controller) {
            controller.updateStateFromEvt(evt, ip);
        }
    },
    /**
     * Send a message to an IFrame
     * @param {String} ip
     * @param {String} message
     */
    postMessage: function(ip, message) {
        var iframe = this.iframes[ip],
            iframeEl;
        if(iframe) {
            iframeEl = Ext.DomQuery.selectNode('iframe', iframe.el.dom);
            iframeEl.contentWindow.postMessage(message, 'http://'+ip);
        }
    },
    /**
     * Iterates entire store and populates iframes from its records
     * @param store data store container 'ip-address' field
     */
    populateIFramesFromStore: function(store) {
        var iter,
            record;
        
        for(iter = 0; iter < store.getCount(); iter+=1 ) {
            record = store.getAt(iter);
            this.createIFrameFromRecord(record);
        }
    }
    
});
