/**
 * Proxy for saving and retreiving settings from the unit
 *
 * 
 */
Ext.define('Muc.proxy.MucP', {
    extend: 'Ext.data.proxy.Client',
    alias: 'proxy.mucproxy',

    /**
     * @cfg {Object} data
     * Optional data to pass to configured Reader.
     */

    constructor: function(config) {
        this.callParent([config]);
        //ensures that the reader has been instantiated properly
        this.setReader(this.reader);
    },
    
    /**
     * @private
     * 
     */
    updateOperation: function(operation, callback, scope) {
        console.log("updateOperation", operation.action)
        var i = 0,
            recs = operation.getRecords(),
            len = recs.length;
            
        for (i; i < len; i++) {
            recs[i].commit();
        }
        operation.setCompleted();
        operation.setSuccessful();
        
        Ext.callback(callback, scope || this, [operation]);
    },
    
    /**
     * 
     * @param {Ext.data.Operation} operation The Operation to perform
     * @param {Function} callback Callback function to be called when the Operation has completed (whether
     * successful or not)
     * @param {Object} scope Scope to execute the callback function in
     * @method
     */
    create: function() {
        console.log("create");
        this.updateOperation.apply(this, arguments);
    },
    
    /**
     * 
     * @param {Ext.data.Operation} operation The Operation to perform
     * @param {Function} callback Callback function to be called when the Operation has completed (whether
     * successful or not)
     * @param {Object} scope Scope to execute the callback function in
     * @method
     */
    update: function() {
        console.log("update");
        this.updateOperation.apply(this, arguments);
    },
    
    /**
     * 
     * @param {Ext.data.Operation} operation The Operation to perform
     * @param {Function} callback Callback function to be called when the Operation has completed (whether
     * successful or not)
     * @param {Object} scope Scope to execute the callback function in
     * @method
     */
    destroy: function() {
        console.log("destroy");
        this.updateOperation.apply(this, arguments);
    },

    /**
     * Reads data from the configured {@link #data} object. Uses the Proxy's {@link #reader}, if present.
     * @param {Ext.data.Operation} operation The read Operation
     * @param {Function} callback The callback to call when reading has completed
     * @param {Object} scope The scope to call the callback function in
     */
    read: function(operation, callback, scope) {
        console.log("read: ", operation)
        var me = this,
            reader = me.getReader(),
            result,
            doRead;
            

        doRead = function(data) {

            result = reader.read(data);
            Ext.apply(operation, {
                resultSet: result
            });

            operation.setCompleted();
            operation.setSuccessful();
            Ext.callback(callback, scope || me, [operation]);
        };
        
        Ext.Ajax.request({
            url: '/muc.json',
            
            success: function(response) {
                var jsonData = Ext.JSON.decode(response.responseText);
                doRead(jsonData);
            },
            failure: function() {
                console.log("muc not there");
                doRead({});
            }
        });
    },

    clear: Ext.emptyFn,
    
    /**
     * Save the config to the unit
     * @param {Ext.data.Store} store The store containing data
     * @param {Function} successcb 
     * @param {Function} failurecb 
     */
    saveOnUnit: function(store, successcb, failurecb) {
        var idx,
            listOfRecords = [],
            serialRecords;
        for(idx = 0; idx < store.getCount(); idx+=1) {
            var record = store.getAt(idx);
                listOfRecords.push(record.data);
        }
        serialRecords = Ext.JSON.encode(listOfRecords);
        Ext.Ajax.request({
            url: "/cgi-bin/writemuc",
            method: 'POST',
            jsonData: serialRecords,
            success: function() {
                successcb.call(null, arguments);
            },
            failure: function() {
                failurecb.call(null, arguments);
            }
        })
    }
});
