/**
 * This reader converts Barco's UDP discovery messages into a javascript objects
 *  Filters based on type of unit
 */
Ext.define('Muc.reader.DiscoveryR',{
    extend: 'Ext.data.reader.Json',
    alias: 'reader.discoveryR',
    root: '',    
    /**
     * @cfg {String} unitType
     *  type of unit to filter the responses for, ex: "ImagePRO-II"
     */
    
    /**
     * Handle the raw response data
     * @param {Object} response
     */
    getResponseData: function(response) {
        
        var data = [], error, discoveredItems;
 
        try {

            discoveredItems = response.responseText.split('\n');
            //"type=HDX W14;hostname=R&D's HDX;ip-address=172.16.2.12;mac-address=00:04:a5:fe:02:80;"
            Ext.Array.forEach(discoveredItems, function(item) {
                var idx, 
                    attrs = item.split(';'),
                    strRecord,
                    record = null;
                
                for(idx = 0; idx < attrs.length; idx += 1) {
                    strRecord = attrs[idx].split('=');
                    if(strRecord.length > 1) {
                        //["type", "HDX W14"]
                        record = record || {};
                        record[strRecord[0]] = strRecord[1];
                    }
                }
                if(record !== null && record.hostname === "ImagePRO-II") {
                    data.push(record);
                }
                
            });
            return this.readRecords(data);
        } catch (ex) {
            error = new Ext.data.ResultSet({
                total  : 0,
                count  : 0,
                records: [],
                success: false,
                message: ex.message
            });

            this.fireEvent('exception', this, response, error);

            Ext.Logger.error('Unable to parse the Data returned by the server');

            return error;
        }
    }

});


