Ext.ns('barco');

barco.Comet = Ext.extend(Object, {
    connectionLost: false,
    anticipatedUpdates: null,
    constructor: function() {
        "use strict";
        this.getNumberRequests();
    },
    getNumberRequests: function() {
        "use strict";
        
        var totalRequests, 
            createTimer,
            me = this,
            refreshTimer = null,
            originalMiss = null,
            timeToRetry = null;
        
        barco.utils.ajaxRequest({
            method: 'GET',
            url: '/updatestotal',
            failedRequestOk: true,
            scope: me,
            success: function(resp) {
                if(me.connectionLost) {
                    barco.modelProxy.load(function() {
                        console.log('connection back');
                        me.connectionLost = false; 
                    });
                    return;
                }

                totalRequests = parseInt(resp.responseText, 10);
                if(me.anticipatedUpdates === null) {
                    me.anticipatedUpdates = totalRequests;
                    me.waitForChange();
                    
                } else if(me.anticipatedUpdates !== totalRequests) {
                    timeToRetry = new Date().getTime();
                    console.error('update missed. me.anticipatedUpdates ' + me.anticipatedUpdates + ' but got ' + resp.responseText);
                    originalMiss = timeToRetry;
                    
                    if(refreshTimer === null) {
                        createTimer = function(intrvl) {
                            var redo = createTimer;
                            refreshTimer = setTimeout(function() {
                                refreshTimer = null;
                                if((timeToRetry - originalMiss) < 3000) {
                                    //wait for radio silence
                                    //but increment the last miss time
                                    timeToRetry = new Date().getTime();
                                    console.error('TOO SOON');
                                    refreshTimer = null;
                                    redo(1000);
                                } else {
                                    console.log("ALL SYNCED");
                                    barco.modelProxy.load();
                                }

                            }, intrvl);
                        };
                        createTimer(3000);
                    } 
                    me.anticipatedUpdates = totalRequests;
                } /*else {
                    console.log("OK JACK " + totalRequests);
                }*/
            },
            failure:function() {
                if(me.connectionLost === false) {
                    console.log('connection lost');
                }
                me.connectionLost = true;
            }
        });
    },
    waitForChange: function() {
        "use strict";
        var requestTime = new Date().getTime(),
            me = this,
            delayF = function() {
                me.waitForChange();
            };
        this.getNumberRequests();
        barco.utils.ajaxRequest({
            method: 'GET', 
            failedRequestOk: true,
            url:'/update', 
            scope: me,
            verbose: false,
            timeout: 15E3,
            success: function(resp) {
                me.anticipatedUpdates += 1;
                var rusty = resp.responseXML;
                if(!rusty) {
                    console.error(resp);
                    window.rusty = resp;
                } else {
                    barco.modelProxy.importNodes(barco.utils.findAllXmlNodeLeafs(rusty.childNodes[0]));
                }
                me.waitForChange();
            },
            failure: function(resp) {
                var now = new Date().getTime();
                if( (now - requestTime) > 100) { //normal timeout
                    me.waitForChange();
                }
                else {
                    //we probably lost connection to server, not a timeout for update
                    //add a delay before requesting again or browser gets hammered.
                    me.connectionLost = true;
                    setTimeout(delayF, 1000);
                }
            }
        });
    }
});
