
/****
 * The purpose of this component is to fix a bug in Sencha where the listPanel 
 *  for the select field continues to block events from other components 
 *  after it is hidden. 
 *  
 *  And also to add 'Page Up', 'Page Down' buttons.
 *  
 ****/
barco.SelectField = Ext.extend(Ext.form.Select, {
    showComponent: function() {
        "use strict";
        var listPanel, picker, name, value, index;

        if (Ext.is.Phone) {
            picker = this.getPicker();
            name   = this.name;
            value  = {};

            value[name] = this.getValue();
            picker.show();
            picker.setValue(value);
        }
        else {
            listPanel = this.getListPanel();
            index = this.store.findExact(this.valueField, this.value);

            listPanel.showBy(this.el, 'fade', false);
            if(index != -1) {
                listPanel.down('#list').getSelectionModel().select(index, false, true);
            }
            //otherwise nothing selected
            listPanel.el.applyStyles("display: block");
        }
    },
    onListSelect: function() {
        "use strict";
        barco.SelectField.superclass.onListSelect.apply(this, arguments);
        if(Ext.is.Phone !== true) {
            this.listPanel.el.applyStyles("display: none");
        }
    },
    getListPanel: function() {
        "use strict";
        var me = this;
        if (!this.listPanel) {
            this.listPanel = new Ext.Panel({
                floating         : true,
                stopMaskTapEvent : false,
                hideOnMaskTap    : true,
                width            : 240,
                cls              : 'x-select-overlay',
                scroll           : 'vertical',
                dockedItems: [{
                    xtype: 'toolbar',
                    dock: 'top',
                    items: [{
                        text: 'Page Up',
                        width: 90,
                        handler: function() {
                            //height minus the height of the toolbar.
                            var newOffset = me.listPanel.getHeight() - me.listPanel.dockedItems.items[0].getHeight() - 10,
                                oldOffset = me.listPanel.scroller.offset.y;
                            me.listPanel.scroller.scrollView.showIndicators();
                            if(oldOffset < -newOffset) {
                                me.listPanel.scroller.scrollBy({x:0, y:newOffset}, true);
                            } else {
                                //keep from overflowing
                                me.listPanel.scroller.scrollTo({x:0, y:0}, true);
                            }

                        }

                    }, {
                        xtype: 'spacer'
                    },{
                        text: 'Page Down',
                        handler: function() {
                            var newOffset = me.listPanel.getHeight() - me.listPanel.dockedItems.items[0].getHeight() - 10,
                                oldOffset = me.listPanel.scroller.offset.y;
                            me.listPanel.scroller.scrollView.showIndicators();
                            if((-oldOffset + newOffset + me.listPanel.getHeight()) > me.listPanel.scroller.size.height) {
                                //keep from overflowing
                                me.listPanel.scroller.scrollTo({x:0, y:me.listPanel.scroller.size.height}, true);
                            } else {
                                me.listPanel.scroller.scrollBy({x:0, y:-newOffset}, true);
                            }
                        }
                    }]
                }],
                items: {
                    xtype: 'list',
                    store: this.store,
                    itemId: 'list',
                    scroll: false,
                    itemTpl : [
                        '<span class="x-list-label">{' + this.displayField + '}</span>',
                        '<span class="x-list-selected"></span>'
                    ],
                    listeners: {
                        select : this.onListSelect,
                        scope  : this
                    }
                }
            });
            me.listPanel.on('afterrender', function(){
                barco.utils.panelScrollMixin(me.listPanel);
            });
        }
        return this.listPanel;
    }    
});

Ext.reg('barcoselectfield', barco.SelectField);

barco.InputFormatSelector = {
    xtype: 'barcoselectfield',
    id: 'input-select',
    name: barco.tags.Name,
    componentStore: "InputFormatStore",
    noXMLWrite: true,
    label: 'Input Format',
    options: [{
        text: '', 
        value: ''
    }],
    listeners: {
        change: function (component, value) {
            "use strict";
            var node = barco.utils.xmlselectnode(barco.tags.ActiveConn),
                currInput = parseInt(node.textContent, 10);
            
            // DVI input requires specifying the input type. 
            if (currInput == 0) {            
                barco.utils.ascii('vfs --con '+String(currInput)+' --itype '+barco.utils.dviInputType()+' --new "' + component.store.getAt(value).data.text + '"');
            }           
            else {
                barco.utils.ascii('vfs --con '+String(currInput)+' --new "' + component.store.getAt(value).data.text + '"');
            }			          
            //barco.utils.ascii('resi ' + (currInput + 1) + ' ' + value);
        },
        afterrender: function(component){
            "use strict";            
            barco.modelProxy.addCallback({
                scope: component, 
                storename: 'InputFormatStore',
                func: barco.setResolutionByIndex
            });
        }
    }
};

barco.OutputFormatSelector = {
    xtype: 'barcoselectfield',
    id: 'output-select',
    name: barco.tags.Name,
    componentStore: 'OutputFormatStore',
    noXMLWrite: true,
    label: 'Format',
    index: 0,
    options: [{
        text: '', 
        value: ''
    }],
    listeners: {
        change: function (component, value) {
            "use strict";
            barco.utils.ascii('vfs --och '+String(component.index)+' --new "' + component.store.getAt(value).data.text + '"');
            if(barco.utils.is3DMode()){
                var x3DPacking = Ext.getCmp('output-select-3d-packing');
                if(component.store.getAt(value).data.text.indexOf("FP") !== -1){
                    x3DPacking.setFPOptions();
                }else{
                    if(barco.utils.isDualPixelMode()){
                        x3DPacking.setLROptions(false, true);
                    }else{
                        x3DPacking.setLROptions(false);
                    }
                }
            }
        },
        afterrender: function(component){
            "use strict";
            barco.modelProxy.addCallback({
                scope: component, 
                storename: 'OutputFormatStore',
                func: barco.setResolutionByIndex
            });
        }
    }
};

barco.ColorBalance = Ext.extend(Ext.form.FormPanel, {
    scroll: 'verticle',
    itemDefaults: {},
    items: 
    [{
        xtype: 'fieldset',
        defaults: {
            cls: 'needwheel',
            labelWidth: '35%',
            xtype: 'barcoslider'
        },
        items: [
            {
                name: 'ContrR',
                label: 'Red Contrast'
            }, {
                name: 'ContrG',
                label: 'Green Contrast'
            }, {
                name: 'ContrB',
                label: 'Blue Contrast'
            }, {
                name: 'BrightR',
                label: 'Red Brightness'
            }, {
                name: 'BrightG',
                label: 'Green Brightness'
            }, {
                name: 'BrightB',
                label: 'Blue Brightness'
            }, {
                name: 'Hue',
                label: 'Hue'
            }, {
                name: 'Sat',
                label: 'Saturation'
            }]
    }],
    constructor: function (config) {
        "use strict";
        var me = this;
        this.items[0].items.forEach(function(el) {
            if(el.name == 'Hue')
            {
                el.minValue = me.hueMinMax[0];
                el.maxValue = me.hueMinMax[1];
            }
            else
            {
                el.minValue = me.colorMinMax[0];
                el.maxValue = me.colorMinMax[1];
            }
        });
        Ext.apply(this.items[0].defaults, this.itemDefaults);
        barco.modelProxy.listeners.addDefaultListeners(this);
        barco.ColorBalance.superclass.constructor.call(this, config);
    }
});

barco.TestPatterns = Ext.extend(Ext.form.FormPanel, {
    id: 'Test Patterns',
    scroll: 'verticle',
    items: [{
        xtype: 'fieldset',
        defaults: {
            labelWidth: '35%'
        },
        items: [
            {
                xtype: 'selectfield',
                id: 'Test-Pattern-Select-Field',
                noXMLWrite: false,
                name: barco.tags.TestPatternMode,
                label: 'Type',
                store: barco.TestPatternTypes, //array of options 
                componentStore: 'TestPatternStore'
            }, {
                xtype: 'togglefield',
                name: barco.tags.RasterBoxMode,
                label: 'AOI Raster Box'
            }, {
                xtype: 'togglefield',
                name: barco.tags.DiagMotion,
                label: 'Diagonal Motion'
            }
        ]
    }],
    index: 0,
    changeOutputChannel: function(chan){
        "use strict";
        if(chan != 0 && chan != 1){
            return;
        }
        this.index = chan;
        //this.setTestPatternType(null, barco.modelProxy.stores.TestPatternModeStore);
        barco.modelProxy.genericCallBack.call(this, null, barco.modelProxy.stores.TestPatternStore);
    },/*
    //the purpose of this is to set the value by index of the test patterns when it loads.
    //that way the check mark is on the correct option.
    setTestPatternType: function(rawxml, store) {
        "use strict";
        var selectfield, index;
        if(store.data.items.length > 0) {
            selectfield = Ext.getCmp('Test-Pattern-Select-Field');
            index = selectfield.store.find("text", store.data.items[this.index].data[barco.tags.TestPatternMode], 0, false, true, true);
            if(index == -1) {
                console.error("Unable to find index for test pattern");
            }
            selectfield.setValue(index);
        }
    },*/
    DisableControlForMinDelay: function() {
        "use strict";    
        var isMinDelay, i, length; 
        length = Ext.getCmp('Test Patterns').fields.length;
        isMinDelay = barco.utils.isMinDelayMode();
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('Test Patterns').fields.items[i].setDisabled(isMinDelay);                                                
        }               
    },    
    constructor: function (config) {
        "use strict";
        barco.modelProxy.listeners.addDefaultListeners(this);
        barco.TestPatterns.superclass.constructor.call(this, config);
        barco.modelProxy.enableGenericFormCallback(this, 'TestPatternStore');
        
        this.index = barco.evtManager.ChannelChange;
        
        this.on("show", function(){
            this.changeOutputChannel(this.index); // to update components
            this.DisableControlForMinDelay();
        });
      /*  
        barco.modelProxy.addCallback({
            storename: 'TestPatternModeStore',
            scope: this,
            func: this.setTestPatternType
        });*/
        barco.evtManager.on("ChannelChange", this.changeOutputChannel, this);
    }
});

barco.ListItem = Ext.extend(Ext.Component, {
    baseCls: 'listitem',
    pressedCls: 'selectedlistitem',
    renderTpl: [
        '<div class="listitemtext">{description}</div>',
        '<span class="listitemvalue">{value}</span>'
    ],
    initialValue: "Value",
    description: "Description",
    initRenderData: function () {
        "use strict";
        barco.ListItem.superclass.initRenderData.apply(this, arguments);
        Ext.applyIf(this.renderData, {
            description: this.description,
            value: this.initialValue
        });
        return this.renderData;
    },
    afterRender: function () {
        "use strict";
        barco.ListItem.superclass.afterRender.call(this);
        this.descriptionEl = Ext.DomQuery.selectNode('[class="listitemtext"]', this.el.dom);
        this.valueEl = Ext.DomQuery.selectNode('[class="listitemvalue"]', this.el.dom);
    },
    initEvents: function () {
        "use strict";
        barco.ListItem.superclass.initEvents.call(this);
        this.mon(this.el, {
            scope: this,
            
            tap: this.onPress
        });
    },
    onPress: function () {
        "use strict";
        this.fireEvent('change', this, true);
    },
    setValue: function (val) {
        "use strict";
        this.valueEl.textContent = String(val);
    }
});

Ext.reg('barcolistitem', barco.ListItem);

barco.CommitHistory = Ext.extend(Ext.Panel, {
    cls: 'barcolist',
    id: 'commithistory',
    scroll: 'verticle',
    maxItemsAllowed: 20,
    
    initComponent: function () {
        "use strict";
        this.itemsSelected = 0;
        this.revertButton = new Ext.Button({
            text: 'Revert to Selected', 
            disabled: true,
            listeners: {
                tap: function (btn, evt) {
                    var idx, store,
                        me = this,
                        itemList = this.items.items,
                        itemsToRemove = [];
                    itemList.forEach(function (item, idx) {
                        if (item.pressed && !item.hidden) {
                            itemsToRemove.push(item);
                            store = barco.modelProxy.stores[item.storename];
                            if(barco.modelProxy.stores[item.storename] !== undefined){
                                barco.modelProxy.changeStoreValue(store, item.storeindex, item.storetag, item.oldValue);
                            }
                        }
                    });
                    itemsToRemove.forEach(function (item) {
                        for (idx = 0; idx <  me.items.items.length; idx += 1) {
                            if (item ===  me.items.items[idx]) {
                                me.remove(idx);
                            }
                        }
                    });
                    me.itemsSelected = 0;
                    btn.disable();
                    this.doLayout();
                },
                scope: this
            }
        });
        this.dockedItems = this.dockedItems || [];
        this.dockedItems.unshift(new Ext.Toolbar({
            dock: 'top',
            items: [this.revertButton]
        }));
        barco.CommitHistory.superclass.initComponent.call(this, arguments);
        this.addListener('beforeshow', this.showHideInputChanges, this);
        
    },
    initEvents: function() {
        "use strict";
        var me = this;
        barco.CommitHistory.superclass.initEvents.call(this);
        barco.utils.panelScrollMixin(this);
    },
    showHideInputChanges: function(fromInputChange) {
        "use strict";
        var itemList = this.items.items,
            me = this,
            currInput = barco.utils.xmlselectnode(barco.tags.ActiveConn).textContent;
        itemList.forEach(function(item, idx) {
            var savedInput = item.savedInput;
            if( savedInput && (savedInput != currInput)) {
                item.hide();
            } else {
                item.show();
            }
            if(fromInputChange===true)
            {
                item.removeCls(item.pressedCls);
                item.pressed = false;
            }
        });
        if(fromInputChange===true)
        {
            this.itemsSelected = 0;
            this.revertButton.disable();
        }
    },
    /*
     * Register a change that can later be reverted through a list item.
     * @param: component
     * @param: description
     * @param: oldValue
     * @param: newValue
     * @param: savedInput, optional: if change is tied to specific input
     */
    addChange: function (storename, storeindex, storetag, oldValue, newValue, savedInput, description) {
        "use strict";
        var totalAdded = this.items.length,
            time = new Date(),
            item;
        description = description + ", new value: " + newValue + ", Date: " + time.format("h:i:s a");
        if(storeindex !== undefined){
            description += (" - Ch. " + (storeindex==0?"A":"B"));
        }
        item = new barco.ListItem({
            description: description,
            initialValue: oldValue
        });
        item.savedInput = savedInput;
        item.oldValue = oldValue;
        item.storename = storename;
        item.storeindex = storeindex || 0;
        item.storetag = storetag;
        
        if(this.itemsSelected > 0)
        {
            item.itemsSelected += 1;
            item.addCls(item.pressedCls);
            item.pressed = true;
        }
        item.on('change', function (cmpt, newv) {
            var index = this.items.indexOf(cmpt), idx, entry;
            this.itemsSelected = 0;
            for(idx=0; idx<this.items.length; idx+=1)
            {
                entry = this.items.items[idx];
                if(idx<=index)// && !entry.hidden)
                {
                    this.itemsSelected += 1;
                    entry.addCls(entry.pressedCls);
                    entry.pressed = true;
                }
                else
                {
                    entry.removeCls(entry.pressedCls);
                    entry.pressed = false;
                }
            }
            this.revertButton.enable();
            
        }, this);
        this.items.insert(0, item);
        if(totalAdded >= this.maxItemsAllowed) {
            this.remove(totalAdded);
        }

        this.doLayout();
    }
});


barco.errorPanel = new Ext.Panel({
    floating: true,
    centered: true,
    id: 'errorPanel',
    width: '320px',
    modal: true,
    hideOnMaskTap: false,
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'bottom',
        items: [{
            text: 'Reload Page',
            listeners: {
                tap: function() {
                    "use strict";
                    window.location.reload();
                }
            }
        }, {
            xtype: 'spacer'
        },{
            text: 'Clear Errors',
            handler: function() {
                "use strict";
                Ext.getCmp('errorButton').resetErrors();
                barco.utils.ajaxError.ignoreErrors = true;
                barco.errorPanel.hide();
            }
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        title: 'Error'
    }],
    items: [{
        xtype: 'panel',
        style: 'padding: 15px;',
        html: "<p>Communication errors have occurred. You can either <strong>reload</strong> the page or <strong>clear and ignore</strong> future errors.</p>"
    }]
});


barco.ModalPopup =  Ext.extend(Ext.Panel, {
    //user settings
    top: 400,
    title: 'Error',
    content: 'test', //can be html
    moreInfoOn: true,
    moreInfo: 'more info', //can be html
    boptionsOn: true,
    boptions: ["no", "yes"],
    
    //fixed
    width: '320px',
    floating: true,
    modal: true,
    hideOnMaskTap: false,
    initComponent: function() {
        "use strict";
        var itms = [], me = this, 
            lstnr =  function() {
                me.hide();
                if(me.acceptedCallback){
                    me.acceptedCallback(this.acceptedResult);
                }
            };
        if(!this.boptionsOn){
            this.boptions = ["", "Ok"];
        }
        if(this.boptionsOn){
            itms.push({
                text: this.boptions[0],
                handler: lstnr,
                acceptedResult: false
            });
        }
        itms.push({xtype: 'spacer'},{
            text: this.boptions[1],
            handler: lstnr,
            acceptedResult: true
        });
        if(!this.boptionsOn){
            itms.push({xtype: 'spacer'});
        }
        this.bottomToolbar = new Ext.Toolbar({
            dock: 'bottom',
            items: itms
        });
        
        itms = [];
        if(this.moreInfoOn){
            itms.push({
                xtype: 'spacer'
            },
            {
                xtype: 'togbutton',
                text: '?',
                listeners: {
                    tap: function() {
                        this.ownerCt.ownerCt.toggleMoreInfo(this.toggleState);
                    }
                }
            });
        }
        this.topToolbar = new Ext.Toolbar({
            xtype: 'toolbar',
            dock: 'top',
            title: this.title,
            items: itms
        });
        
        this.dockedItems = this.dockedItems || [];
        this.dockedItems.push(this.bottomToolbar);
        this.dockedItems.push(this.topToolbar);
        
        this.items = this.items || [];
        this.items.push( new Ext.Panel({
            style: 'padding: 15px;',
            html: '<p>'+this.content+'</p>'+
                  '<div class="moreinfo x-hidden-display">'+this.moreInfo+'</div>' //use template?
        }));
        
        barco.ModalPopup.superclass.initComponent.call(this);
        //this.down("panel").update(
        //        '<div class="moreinfo x-hidden-display">'+this.moreInfo+'</div>'); 
    },
    constructor: function(config) {
        "use strict";
        barco.ModalPopup.superclass.constructor.call(this, config);
    },
    toggleMoreInfo: function(toggle){
        "use strict";
        var fun = (toggle?'remove':'add');
        Ext.DomQuery.selectNode(".moreinfo",this.getEl().dom).classList[fun]('x-hidden-display');
        this.down("panel").doComponentLayout();
    },
    listeners:{
        show:function(){
            "use strict";
            var x = (Ext.Element.getViewportWidth() / 2) - (this.getWidth() / 2);
            this.setPosition(x,this.top);
        },
        beforehide:function(){
            "use strict";
            if(this.moreInfoOn){
                Ext.DomQuery.selectNode(".moreinfo",this.getEl().dom).classList.add('x-hidden-display');
                this.down("togbutton").resetToggle();
            }
            this.down("panel").doComponentLayout();
        }
    }
});

Ext.reg('barcomodalpopup', barco.ModalPopup);

barco.m1 = new barco.ModalPopup({
    id:"mmm1",
    content: "garpalapaca",
    moreInfo: "Make sure the monkey is in the cage and that the independents are watching the news"
});


