/* 
 * Form button is a form field containing a button on the right and 
 * another element(select, textfield) on the left
 */
barco.form.ButtonField = Ext.extend(Ext.form.Text, {
    buttonlabel: 'label',
    placeHolder: "placeholder",
    useClearIcon: true,
    noTextLabel: false,
    label: 'label',
    disableButtonOnEmpty: false,
    componentCls: 'barcobuttonfield',
    useMask: false,
    validateInput: function(input) {
        "use strict";
        return (/^[a-z,0-9]{0,20}$/i).test(input);
    },
    renderTpl: [
        '<tpl if="label">',
            '<div class="x-form-label">',
                '<tpl if="noTextLabel==false">', //option hide label
                    '<span style="float: none">{label}</span>',
                '</tpl>',
                '<div style="-webkit-box-orient: horizontal; -webkit-box-pack:center; display: -webkit-box">',
                    '<div class="x-button x-button-normal">',
                        '<span class="x-button-label">{buttonlabel}</span>',
                    '</div>',
                '</div>',
            '</div>',
        '</tpl>',
          '<tpl if="fieldEl"><div class="x-form-field-container">',
            '<tpl if="useMask!==true">',
            '<input id="{inputId}" type="{inputType}" name="{name}" class="{fieldCls}" style="height: {inputFieldHeight}" ',
                '<tpl if="tabIndex">tabIndex="{tabIndex}" </tpl>',
                '<tpl if="placeHolder">placeholder="{placeHolder}" </tpl>',
                '<tpl if="style">style="{style}" </tpl>',
                '<tpl if="minValue != undefined">min="{minValue}" </tpl>',
                '<tpl if="maxValue != undefined">max="{maxValue}" </tpl>',
                '<tpl if="stepValue != undefined">step="{stepValue}" </tpl>',
                '<tpl if="autoComplete">autocomplete="{autoComplete}" </tpl>',
                '<tpl if="autoCapitalize">autocapitalize="{autoCapitalize}" </tpl>',
                '<tpl if="autoFocus">autofocus="{autoFocus}" </tpl>',
            '/>',
            '</tpl>',
            '<tpl if="useMask">',
                '<div class="x-field-mask">',
                '</div>',
            '</tpl>',
            '</div></tpl>',
        '<tpl if="useClearIcon"><div class="x-field-clear-container"><div class="x-field-clear x-hidden-visibility">&#215;</div><div></tpl>'
    ],
    constructor: function(config) {
        "use strict";
        barco.form.ButtonField.superclass.constructor.call(this, config);
    },
    initComponent: function() {
        "use strict";
        barco.form.ButtonField.superclass.initComponent.call(this);
        this.addEvents('buttonclick');
    },
    initRenderData: function() {
        "use strict";
        var inputFieldHeight = '80px';
        barco.form.ButtonField.superclass.initRenderData.call(this);
        if(this.noTextLabel) {
            inputFieldHeight='56px';
        }
        Ext.applyIf(this.renderData, {
            buttonlabel: this.buttonlabel,
            placeHolder: this.placeHolder,
            noTextLabel: this.noTextLabel,
            useClearIcon: true,
            inputFieldHeight: inputFieldHeight
        });
        return this.renderData;
    },
    afterRender: function() {
        "use strict";
        var me = this;
        barco.form.ButtonField.superclass.afterRender.call(this);
        this.button = Ext.get(Ext.DomQuery.selectNode('.x-button', this.el.dom));
        this.textfield = Ext.get(Ext.DomQuery.selectNode('input', this.el.dom));
        this.button.disable = function () {
            me.button.addCls('x-item-disabled');
        };
        this.button.enable = function () {
            me.button.removeCls('x-item-disabled');
        };
        if(this.disableButtonOnEmpty) {
            this.button.disable();
        }
        this.button.on('touchstart', function(evt, el) {
            if(me.button.hasCls('x-item-disabled')) {
                return;
            }
            me.button.addCls('x-button-pressed');
            me.fireEvent('buttonclick', me);
        });
        this.button.on('touchend', function(evt, el) {
            me.button.removeCls('x-button-pressed');
        });

        if(this.textfield !== null) {
            this.textfield.on('input', this.doValidation, this);
        }
    },
    doValidation: function() {
        "use strict";
        if(this.validateInput(this.textfield.dom.value) === false) {
           this.textfield.setStyle('background-color', '#FF5555');
           this.button.disable();

        } else {
           this.textfield.setStyle('background-color', 'white');
           this.button.enable();
        }
        if(this.disableButtonOnEmpty && this.textfield.dom.value === '') {
           this.button.disable();
        }
    }, 
    onClearIconTap: function() {
        "use strict";
        barco.form.ButtonField.superclass.onClearIconTap.call(this);
        this.doValidation();
    }

});

Ext.reg('barcobuttonfield', barco.form.ButtonField);

