Ext.ns('barco');

/* **********************
 * The modelProxy get's changes from the server and distributes it to the 
 * component datastores. It has helper functions to apply changes in a generic way. 
 * A component can register via barco.modelProxy.addCallback with config object 
 * containing: 
 *      1) func
 *      2) scope
 *      3) storename
 * **********************/
barco.modelProxy = function() {
    "use strict";
}; //not a constructor

barco.modelProxy.create = function () {
    "use strict";
    this.created = true;
    this.callbacks = [];
    this.stores = {};
    this.cbproto = function (fun, scope, storename) {
        this.fun = fun;
        this.scope = scope;
        this.storename = storename;
    };
    
    barco.modelProxy.getResolutionsAndPopulate();
    
    this.load();
    var storeList = barco.dataStores.stores,
        storeIdx,
        storeConfig;

    for(storeIdx = 0; storeIdx < storeList.length; storeIdx += 1) {
        storeConfig = storeList[storeIdx];
        Ext.applyIf(storeConfig, barco.dataStores.defaults);
        console.log("store = " + storeConfig.barconame);
        this.stores[storeConfig.barconame] = new Ext.data.Store(storeConfig);
    }
};

barco.modelProxy.getResolutionsAndPopulate = function(successCallback) {
    "use strict";
    barco.utils.getResolutions(function(resArray) {
        var options = [], idx;
        for(idx = 0; idx < resArray.length; idx += 1) {
            options.push({
                text: resArray[idx], 
                value: idx
            });
        }
        
        Ext.getCmp('input-select').setOptions(options, false);
        Ext.getCmp('input-select-mainui').setOptions(options, false); 
        Ext.getCmp('output-1-select').setOptions(options, false);
        Ext.getCmp('output-2-select').setOptions(options, false);
        Ext.getCmp('output-select-mainui-1').setOptions(options, false);
        Ext.getCmp('output-select-mainui-2').setOptions(options, false);
        if(successCallback!==undefined){
            successCallback();
        }
    }, this);
};


barco.modelProxy.load = function(cbk) {
    "use strict";
    barco.utils.store.load({
        action: 'write',
        callback: cbk
    });
};

barco.modelProxy.addCallback = function (opts) {
    "use strict";
    var idx, store;
    opts.func = opts.func || barco.modelProxy.genericCallBack;
    if(this.created === undefined) {
        this.create();
    }
    for(idx = 0; idx < this.callbacks.length; idx+=1) {
        //avoid duplicates
        if( (this.callbacks[idx].scope == opts.scope) &&
            (this.callbacks[idx].storename == opts.storename) && 
            (this.callbacks[idx].fun === opts.func)){
            return;
        }
    }

    this.callbacks.unshift(new this.cbproto(opts.func, opts.scope, opts.storename));
    store = this.stores[opts.storename];
    if(store === undefined && opts.storename !== undefined) {
        console.log("Error: store not found " + opts.storename);
    }
};

barco.modelProxy.modelChanged = function() {
    "use strict";
    var idx, obj, store;
    if(this.created === undefined) {
        return;
    }
    
    this.loadSecondary();
    for(idx = 0; idx < this.callbacks.length; idx += 1) {
        obj = this.callbacks[idx];
        store = this.stores[obj.storename];
        obj.fun.call(obj.scope, barco.utils.store.rawxml, store);
    }
};

barco.modelProxy.storeChanged = function(storename) {
    "use strict";
    var idx, obj, store;
    if(this.created === undefined) {
        return;
    }
    //console.log("storeChanged pushing: " + storename);
    for(idx = 0; idx < this.callbacks.length; idx += 1) {
        obj = this.callbacks[idx];
        if(obj.storename==storename) {
            store = this.stores[obj.storename];
            obj.fun.call(obj.scope, barco.utils.store.rawxml, store);
        }
    }
};

barco.modelProxy.getStoreValue = function(searchObj) //searchObj: {store:"storename", tag:"tag"}
{
    "use strict";
    if(searchObj.tag === undefined || searchObj.store === undefined) {
        return undefined;
    }
    var store = this.stores[searchObj.store];
    if(store === undefined ) {
        return undefined;
    }
    if(store.data.items.length) {
        return store.data.items[0].data[searchObj.tag];
    }
    return undefined;
};

barco.modelProxy.genericCallBack = function(rawxml, store) {
    "use strict";
    //console.log("77pushing data " + store.barconame);
    if(this.isVisible() && this.rendered) {
        this.changesLocked = true;
        this.barcostore = store;
        this.loadModel(store.data.items[this.index || 0]);
        this.changesLocked = false;
    }
};

/* pushes changes from hardware to UI component
 *  Only enable to update components generically from XML
 * */
barco.modelProxy.enableGenericFormCallback = function (obj, storename) {
    "use strict";
    
    var func = barco.modelProxy.genericCallBack;

    //push changes to components
    barco.modelProxy.addCallback({
        func: func,
        scope: obj, 
        storename: storename
    });
    obj.on('show', function() {
        func.call(this, this, barco.modelProxy.stores[storename]);
    }, obj);
    
};

barco.modelProxy.loadSecondary = function() {
    "use strict";
    var storeName, store;
    for(storeName in this.stores) {
        if(this.stores.hasOwnProperty(storeName)) {
            store = this.stores[storeName];

            store.getProxy().data = store.selectXmlSpecific(barco.utils.store.rawxml);
            store.load();
        }
    }
};

barco.modelProxy.getStoreRawXml = function(storename, index) {
    "use strict";
    index = index || 0;
    if(storename === undefined) {
        return undefined;
    }
    var store = barco.modelProxy.stores[storename];
    if(store === undefined ) {
        return undefined;
    }
    return store.data.items[index].raw;
};

barco.modelProxy.changeStoreValue = function(store, index, name, newval, ajaxSuccess, returnOldValue, dontUpdateLocal) {
    "use strict";
    var oldValue, tag, xml, node, realval,
        field = store.model.prototype.fields.get(name);
    if(field) {
        tag = (field.mapping !== undefined && field.mapping !== null) ? field.mapping : field.name;
    }
    else {
        tag = name;
    }
    
    xml = store.apply({
        tag: tag, 
        value: newval
    }, index);
    
    barco.utils.ajaxRequest(Ext.apply({request: xml}, ajaxSuccess));
    
    //update the local data
    if(returnOldValue || !dontUpdateLocal){
        node = Ext.DomQuery.selectNode(tag, store.data.items[index].raw);
        if(node !== undefined) {
            oldValue = store.toViewValue(tag, node.textContent, index);
            
            if(!dontUpdateLocal)
            {
                realval = store.fromViewValue(tag, newval, index);
                node.textContent = String(realval);
                store.getAt(index).set(tag, realval);
            }
        }
    }
    return oldValue;
};

/******************************************************************************* 
 * barco.modelProxy.updateComponentLite - utility function to update the value of a 
 * component without running through recent changes. It emulates the drag behavior
 * of a slider. Returns updater object. The workflow for using the returned object is
 *  - lock
 *  - update
 *  - ...
 *  - update 
 *  - finish
 *******************************************************************************/
barco.modelProxy.updateComponentLite = function(component) {
    "use strict";
    var updater = {},
        store,
        componentStore,
        owner = component.ownerCt.ownerCt,
        cmpStoreName = component.componentStore;
       
    if(cmpStoreName !== undefined) {
        componentStore = barco.modelProxy.stores[cmpStoreName];
    }
    store = componentStore || owner.barcostore;

    updater.lock = function() {
        owner.changesLocked = true;
    };
    updater.update = function(value) {
        barco.modelProxy.changeStoreValue(store, owner.index || 0, component.name, value, null, false, true);
        component.setValue(value);
    };
    updater.finish = function() {
        owner.changesLocked = false;
        component.setValue(component.getValue());
    };
    return updater;
};

//generic listener for form with fieldset to create XML message on item change
barco.modelProxy.listeners = {
    
    fieldset:{
        change: function(component, thumb, newval) {
            "use strict";
            if(component.noXMLWrite) {
                return;
            }
            var componentStore = null,
                cmpStoreName = component.componentStore,
                owner = component.ownerCt.ownerCt,
                store,
                oldValue,
                savedInput;
                
            if(cmpStoreName !== undefined) {
                componentStore = barco.modelProxy.stores[cmpStoreName];
            }
            store = componentStore || owner.barcostore;
            //protect from EXT change on render
            if(owner.changesLocked || (owner.barcostore == undefined)) {
                return;
            }
            if(store !== undefined) {
                owner.dragging = false;
                
                if(component.xtype === 'selectfield' || component.xtype === 'barcoselectfield') {
                    newval = component.value;
                } //the multiselect is a little weird as change cb is called differently
                
                oldValue = barco.modelProxy.changeStoreValue(store, owner.index || 0, component.name, newval, 
                        {scope:component, success:component.ajaxChangeSuccess}, true, component.dontUpdateLocal);
                if(oldValue !== undefined) {
                    if(component.oldValueBarco !== undefined ) {
                        oldValue = component.oldValueBarco;
                    }
                }
                //console.log("oldValue, newval " + oldValue + ", " + newval);
                if((oldValue||oldValue==0) && (oldValue != newval)) {
                    
                    if(store.ioType === 'input') {
                        savedInput = barco.utils.xmlselectnode(barco.tags.ActiveConn).textContent;
                    } 
                
                    //console.debug("old:"+String(oldValue));
                    barco.commitHistory.addChange(
                        store.barconame, 
                        owner.index, //if undefined, no "Channel A" text shown
                        component.name,
                        oldValue, 
                        newval, 
                        savedInput,
                        store.barcolabel + " " + component.label);
                }
                if(!component.noafterchange && component.afterchange !== undefined) {
                    component.afterchange(component.getValue());
                }
            }
        },
        drag: function(component, thumb, newval) {
            "use strict";
            component.targetValue = newval;

            if(component.valLast !== undefined && newval == component.valLast) {
                return;
            }
            
            var doIt = function(component) {
                var componentStore = null,
                    cmpStoreName = component.componentStore,
                    owner = component.ownerCt.ownerCt,
                    store,
                    diff;
                owner.dragging = true;
                if(cmpStoreName !== undefined) {
                    componentStore = barco.modelProxy.stores[cmpStoreName];
                }
                store = componentStore || owner.barcostore;
                if(owner.changesLocked) {
                    return;
                }
                if(store !== undefined) {
                    
                    barco.modelProxy.changeStoreValue(store, owner.index || 0, component.name, component.targetValue, 
                            {scope:component, success:component.ajaxChangeSuccess}, false, true);
                    
                    component.startedDrag = true;
                    if(!component.noafterdrag && component.afterdrag !== undefined) {
                        component.afterdrag(component.targetValue);
                    }
                }
                component.valLast = component.targetValue;
            };
            
            
            if(component.throttle === undefined) {
                component.throttle = true;
                //doIt(component)
                component.timerStarted = true;
                setTimeout(function() {
                    delete component.throttle;
                    delete component.timerStarted;
                    doIt(component, newval);
                }, 150);
            }
            else if (component.throttle){
                if(component.timerStarted === undefined) {
                    doIt(component, newval);
                }
            }

        },
        beforechange: function (component, thumb, newvalue, oldvalue) {
            "use strict";
            component.valueBeforeChange = oldvalue;
        }
    },
    
    //should be called before the Ext component constructor
    //Pushes changes from UI to hardware in a generic way
    addDefaultListeners: function(panel) {
        "use strict";
        panel.items.forEach(function(item) {
            if(item.defaults !== undefined) {
                item.defaults.listeners = item.defaults.listeners || {};
                Ext.applyIf(item.defaults.listeners, barco.modelProxy.listeners.fieldset);
            }
        });
    }
};

/**********
 * barco.modelProxy.importNodes - 
 *  @nodes Object containing leafs nodes.node (Element) and nodes.path (query path)
 **********/
barco.modelProxy.importNodes = function(nodes) {
    "use strict";
    nodes.forEach(function(nodeObj) {
        var originalNode = barco.utils.xmlselectnode(nodeObj.path), 
            newnode;
        if(originalNode !== undefined) {
            newnode = document.importNode(nodeObj.node, true);
            originalNode.parentElement.replaceChild(newnode, originalNode);
        }
    });
    
    barco.modelProxy.modelChanged();
};
