
barco.Slider = Ext.extend(Ext.form.Slider, {
    sliderfieldstyle:   "width: " + (Ext.is.Desktop ? '80px;' : '60px;') +
                        "height: 60px;" +
                        "padding: 0px 0px 0px 2px; " +
                        "border: solid 1px black;" +
                        "text-align: right;",

    renderTpl: [
        '<tpl if="label">',
            '<div class="x-form-label"><span>{label}</span></div>',
        '</tpl>',
        '<tpl if="fieldEl">',
            '<input type=number id={valdisp} style="{sliderInputStyle}" step="{stepValue}">',
            '<div id="{inputId}" name="{name}" class="{fieldCls}"',
            '<tpl if="tabIndex">tabIndex="{tabIndex}"</tpl>',
            '<tpl if="style">style="{style}" </tpl>',
        '/></tpl>'
    ],
    constructor: function(config) {
        "use strict";
        barco.Slider.superclass.constructor.call(this, config);
        this.oppositeTag = config.oppositeTag;
    },
    initRenderData: function() {
        "use strict";
        barco.Slider.superclass.initRenderData.apply(this, arguments);
        Ext.applyIf(this.renderData, {
           valdisp: this.renderData.inputId + '-bslider',
           sliderInputStyle: this.sliderfieldstyle,
           stepValue: this.increment
        });
        return this.renderData;
    },
    
    afterRender: function() {
        "use strict";
        this.slidval = Ext.select('input', this.el.dom).elements[0];
        var me = this;
        this.slidval.value = this.getValue();
        
        function inputFieldChange(val) {
            //console.log("Setting slider field value: " + val)
            if(isFinite(val)) {
                me.setValue(val);
            } else {
                this.value = me.getValue();
            }
        }
        
        //register for field updates
        this.slidval.oninput = this.slidval.onchange = function(event) {
            inputFieldChange(event.srcElement.value);
        };
        this.slidval.onmousewheel = function(event) {
            var change = 0;
            change = event.wheelDeltaY > 0 ? change = change+me.increment: change-me.increment;
            inputFieldChange(parseFloat(this.value)+change);
        };
        
        
        //register for slider updates
        this.on('change', function (slider, thumb, newValue, oldValue) {
            slider.setSliderFieldValue(newValue);
        });
        this.on('drag', function(slider, thumb, newValue) {
            slider.setSliderFieldValue(newValue);
        });
        
        barco.Slider.superclass.afterRender.call(this, arguments);
    },

    setSliderFieldValue: function(constrainedVal) {
        "use strict";
        if(this.slidval != null && this.slidval.value != constrainedVal)
        {
            //console.log("+++ " + constrainedVal);
            if(this.increment < 0.1) {
                constrainedVal = parseFloat(constrainedVal).toFixed(2);
            }
            else if(this.increment < 1) {
                constrainedVal = parseFloat(constrainedVal).toFixed(1);
            }
            this.slidval.value = constrainedVal;
            
        }
    },
    setMax: function(value, animationDuration) {
        "use strict";
        var temp = parseInt(value, 10), newValue, thumb;
        if(isNaN(temp))
        {
            return;
        }
        this.maxValue = temp;
        thumb = this.getThumb();
        newValue = this.constrain(thumb.getValue());
        if(newValue != thumb.getValue())
        {
            return this.setValue(newValue);
        }
        this.moveThumb(thumb, this.getPixelValue(newValue, thumb), animationDuration);
        this.doComponentLayout();
        return this;
    },
    setMin: function(value, animationDuration) {
        "use strict";
        var temp = parseInt(value, 10), newValue, thumb;
        if(isNaN(temp))
        {
            return;
        }
        this.minValue = temp;
        thumb    = this.getThumb();
        newValue = this.constrain(thumb.getValue());
        if(newValue != thumb.getValue())
        {
            return this.setValue(newValue);
        }
        this.moveThumb(thumb, this.getPixelValue(newValue, thumb), animationDuration);
        this.doComponentLayout();
        return this;
    }
    
});

Ext.reg('barcoslider', barco.Slider);




barco.AdjustedSlider = Ext.extend(barco.Slider, {
    constructor: function(config) {
        "use strict";
        barco.AdjustedSlider.superclass.constructor.call(this, config);
        this.minmaxTag = config.minmaxTag || "";
    },
    
    adjustMinMax: function(offset, invert){
        "use strict";
        var xml = barco.modelProxy.getStoreRawXml(this.componentStore, this.ownerCt.ownerCt.index),
            sss = Ext.DomQuery.selectNode(this.minmaxTag + ' ' + this.name, xml),
            min, max;
        try {
            min = parseInt(sss.attributes.MIN.textContent, 10);
            max = parseInt(sss.attributes.MAX.textContent, 10);
            if(offset)
            {
                min += offset;
                max += offset;
            }
        } catch (TypeError) { 
            console.error('MIN/MAX attributes missing');
            return;
        } 

        //console.debug(this.id+' ['+min+','+max+']');
        this.ownerCt.ownerCt.changesLocked = true;
        if(this.rendered)
        {
            if(invert)
            {
                this.setMax(-min);
                this.setMin(-max);
            }
            else
            {
                this.setMax(max);
                this.setMin(min);
            }
        }
        this.ownerCt.ownerCt.changesLocked = false;
    }
    
});
    
Ext.reg('barcoadjslider', barco.AdjustedSlider);

