//place jslint hints here. 
/*global Node: true */

Ext.ns('barco');
Ext.ns('barco.utils');


barco.utils.panelScrollMixin = function (panel) {
    "use strict";
     panel.el.dom.onmousewheel = function(event) {
        var change = event.wheelDeltaY/2;
        panel.scroller.scrollView.showIndicators();
        if(change > 0) {
            if(panel.scroller.offset.y < -50) {
                panel.scroller.scrollBy({x:0, y:(change)});
            } else {
                //avoid overflow
                panel.scroller.setOffset({x:0, y:0});
            }
        } else {
            //when a new item is added the me.scroller.size.height  goes to 0
            //update boundary to refresh it.
            panel.scroller.updateBoundary();
            if((-(panel.scroller.offset.y + -50) + panel.getHeight()) < panel.scroller.size.height) {
                panel.scroller.scrollBy({x:0, y:change});

            } else {
                //avoid overflow
                panel.scroller.scrollTo({x:0, y: panel.scroller.size.height});
            }
        }
    };
};


barco.utils.ajaxError = {
    ignoreErrors: false,
    checkForError: function(str) {
        "use strict";
        var status = (this.errorString.hasOwnProperty(str));
        if(status) {
            this.increment();
        }
        return status;
    },
    increment: function() {
        "use strict";
        Ext.getCmp('errorButton').incError();
        this.errorCount = this.errorCount || 0;
        if(this.ignoreErrors !== true) {
            barco.errorPanel.show();
        }
        this.errorCount+=1;
    },
    errorString: {
        '<timeout></timeout>': true, 
        'timeout': true
    }
};


barco.utils.ajaxRequest = function(reqObj)
{
    "use strict";
    reqObj = reqObj || {};
    if(typeof(reqObj) == "string") {
        reqObj = {
            request: reqObj
        };
    }
    //defaults
    reqObj = Ext.apply({
        url: '/ajax',
        method: 'POST',
        timeout: 40000,
        verbose: false,
        failedRequestOk: false
        
    }, reqObj);
    
    var scope = reqObj.scope = reqObj.scope || window,
        url = reqObj.url,
        callFailure;
    return (function(){
        callFailure = function() {
            if(typeof reqObj.failure === 'function'){
                reqObj.failure.apply(scope, arguments);
            }
        };
        Ext.Ajax.request({
            method: reqObj.method,
            url: url,
            timeout: reqObj.timeout,
            rawData:   reqObj.request,
            success: function(  response, opts) {
                var resp, respCode = 0;
                if(typeof reqObj.checkError === 'function')
                {
                    if(reqObj.checkError(response)) {
                        callFailure(response, opts);
                        return;
                    }
                }
                if(barco.utils.ajaxError.checkForError(response.responseText)) {
                    callFailure(response, opts);
                    return;
                }
                resp = Ext.DomQuery.selectNode("Resp", response.responseXML);
                if( resp && parseInt(resp.textContent, 10) < 0)
                {
                    //the probably better thing to do if ImagePro wouldn't send failure numbers with good responses
                    //callFailure(response, opts); 
                    respCode = resp.textContent;
                }
                if(reqObj.verbose) {
                    console.log("ajaxReq Success\n" + response.responseText);
                }
                if(typeof reqObj.success === 'function')
                {
                    reqObj.success.call(scope, response, opts, respCode);
                }
                    
            },
            failure: function(resp, opts) {
                if(reqObj.failedRequestOk === false) {
                    barco.utils.ajaxError.increment();
                }
                if(reqObj.verbose) {
                    console.log("ajaxReq fail " + resp);
                }
                callFailure(resp, opts);
            }
        });
    }());
};


//gets error code from Colugo's xml response 
//example response:
//  <ImagePRO2 Num="0">
//    <CmdType>4</CmdType>
//    <Resp>0</Resp>
//  </ImagePRO2>
barco.utils.getRespErrorCode = function(xmlresp){
    "use strict";
    var leaf = Ext.DomQuery.selectNode("Resp", xmlresp.responseXML);
    return leaf && leaf.textContent;
};

//returns true if xmlresp has a negative response code
barco.utils.checkRespErrorCode = function(xmlresp){
    "use strict";
    return barco.utils.getRespErrorCode(xmlresp) < 0;
};

//queries a node in the xml from the server and updates local xml asynchronously
//if storename is supplied, only that store's change is fired
//otherwise the whole model's change is fired
//calls afterSuccess if provided after successful completion
// @node Element
// @store String or List of store names to update
// @afterSuccess CB
barco.utils.refreshNode = function(node, stores, afterSuccess) {
    "use strict";

    if(typeof stores !== 'object') {
        stores = [stores];
    }
    var recursive,
        isLeaf = node.childElementCount === 0 ? true : false,
        templateObj,
        path,
        indexOfFirstChild,
        req;

    if(isLeaf) { 
        templateObj = barco.tags.reverseTemplate(node.parentElement);
        path = templateObj.path + ' ' + node.nodeName;
        recursive = 0;
    } else {
        templateObj = barco.tags.reverseTemplate(node);
        path = templateObj.path;
        recursive = 1;
    }
    req = templateObj.template.apply({
        value:"<Query>3</Query><Recursive>" + recursive + "</Recursive>"
    });
    
    
    indexOfFirstChild = req.indexOf('<', 1);//search for second <, start tag
    // example of what we want
    // <ImagePRO2 Num="0"><CmdType>7</CmdType><OutputMgr Num="0"><Query>3</Query><Recursive>1</Recursive></OutputMgr></ImagePRO2>
    req = req.substr(0, indexOfFirstChild) + "<CmdType>7</CmdType>" + req.substr(indexOfFirstChild);
    //console.debug(req);
    barco.utils.ajaxRequest({
        request:req, 
        success: function(resp)
        {
            //console.debug(resp.responseXML);
            var nn = Ext.DomQuery.selectNode(path, resp.responseXML),
//                x = console.debug(path),
//                y = console.debug(resp.responseXML),
//                z = console.debug(nn),
                newnode = document.importNode(nn, true),
                oldnode = Ext.DomQuery.selectNode(path, barco.utils.store.rawxml),
                updatedAtleastOne = false;
                    
            oldnode.parentElement.replaceChild(newnode, oldnode);
            stores.forEach(function(storename) {
                if(storename !== undefined)
                {
                    barco.modelProxy.stores[storename].load();
                    barco.modelProxy.storeChanged(storename);
                    updatedAtleastOne = true;
                } 
            });
            //just in case storename wasn't specified
            if(updatedAtleastOne === false) {
                barco.modelProxy.modelChanged();
            }
            if(typeof afterSuccess === 'function'){
                afterSuccess();
            }
        }
    });
};


barco.utils.ascii = function(reqObj) {
    "use strict";
    if(typeof reqObj == "string") {
        reqObj = {
            request: reqObj
        };
    }
    Ext.applyIf(reqObj, {
        url: '/ascii', 
        request: reqObj.request + "\n", 
        verbose: false,
        checkError: function(response)
        {
            //console.debug(response);
            return response.responseText.split(" ")[2] < 0;
        }
    });
    barco.utils.ajaxRequest(reqObj);
};

barco.utils.consoleAscii = function(command) {
    "use strict";
    barco.utils.ascii( { 
        request: command,
        verbose: true,
        checkError: function(resp){
            return false;
        }
    });
};

Ext.regModel('ImagePRO2XML', {
    fields: ['ImagePRO2']
});

barco.utils.store = new Ext.data.Store({
    model: 'ImagePRO2XML',
    proxy: {
        type: 'ajax',
        url : '/ajax',
        reader: {
            type: 'xml',
            record: 'ImagePRO2'
        },
        writer: {
            type: 'xml',
            write: function(req) {
                "use strict";
                return {
                    url: '/ajax',
                    xmlData: '<ImagePRO2 Num="0"><CmdType>7</CmdType><Query>3</Query><Recursive>1</Recursive></ImagePRO2>'
                };
            }
        }
    },
    listeners: {
        load: function(store, records, success){
            "use strict";
            if(success){
                if(barco.utils.ajaxError.checkForError(this.proxy.reader.rawData.documentElement.tagName)) {
                    return;
                }
                this.rawxml = barco.utils.store.data.items[0].raw;
                barco.modelProxy.modelChanged();
            } else {
                barco.utils.ajaxError.increment();
            }
        }
    }
});

barco.utils.store.setLocal = function(selector, value) {
    "use strict";
    if(this.rawxml) {
        var node = Ext.DomQuery.selectNode(selector, this.rawxml);
        node.textContent = value;
        return true;
    }
    return false;
};

barco.utils.xmlselect = function(selector) {
    "use strict";
    return Ext.DomQuery.select(selector, barco.utils.store.rawxml);
};

barco.utils.xmlselectnode = function(selector) {
    "use strict";
    return Ext.DomQuery.selectNode(selector, barco.utils.store.rawxml);
};

barco.utils.selectInput = function(id) {
    "use strict";
    var fun = function()    {
            var inputbar = Ext.getCmp('theinputbar');
            inputbar.setLocked(false);
            barco.commitHistory.showHideInputChanges(true);
            barco.utils.refreshAll();
            //barco.utils.refreshNode(barco.utils.xmlselectnode(barco.tags.FreezeMode), ['InputManagerTopLevel', "SystemStore"]);
        },
        
        inputbar = Ext.getCmp('theinputbar');

    barco.utils.disableCurrentRefreshButton();
    inputbar.setLocked(true);
    barco.utils.ascii(
    {
        request: 'rte '+ (id + 1),
        success: fun,
        failure: fun
    });
};

barco.utils.getResolutions = function (callback, scope) {
    "use strict";
    var dumpFront, dumpBack, finalObj, colon;
    dumpFront = new RegExp("^[ \t\x0d]+");
    dumpBack = new RegExp("[ \t\x0d]+$");
    finalObj = [];
    barco.utils.ajaxRequest({
        url: "/ascii",
        request: 'RESH\n',
        success: function(response, opts) {
            var tmp,
                data = response.responseText.split("RESH")[0];
            data = data.split("\n"); //tokens containing "1: {video format}"
            data.forEach(function(el) {
                colon = el.search(":");
                if( colon >= 0){ //ignore left over junk
                    tmp = el.slice(colon+1);
                    tmp = tmp.replace(dumpFront, ""); //clear leading, trailing spaces
                    tmp = tmp.replace(dumpBack, "");
                    finalObj.push(tmp);
                }

            });
            callback.call(scope, finalObj);
        }
    });
};

barco.utils.addRefreshButton = function() {
    "use strict";
    var additionalItems = [{
        xtype: 'spacer',
        height: '10px'
    }, {
        xtype: 'panel',
        baseCls: 'noclass',
        layout: 'hbox',
        items: [{
            xtype: 'spacer'
        },{
            xtype: 'button',
            text: 'Refresh',
            width: '95px',
            handler: function(btn) {
                var me = this;
                barco.modelProxy.getResolutionsAndPopulate( function(){
                    barco.modelProxy.load( function() {
                        me.enable();
                    });
                });
                
                this.disable();
            }
        }, {
            xtype: 'spacer'
        }]
    }];
    
    return function(panel, config) {
        config = config || {};
        var NewPanel  = Ext.extend(panel, {
            items: panel.prototype.items.concat(additionalItems)
        });
        return new NewPanel(config);
    };
};

//looks for the currently viewable refresh button and if present presses it.
//Otherwise, it refreshes by itself.
barco.utils.refreshAll = function() {
    "use strict";
    var refButton = Ext.getCmp("mainpanel").getActiveItem().down('button[text="Refresh"]');
    if(refButton){
        refButton.callHandler();
    }
    else {
        barco.modelProxy.getResolutionsAndPopulate( function(){
            barco.modelProxy.load();
        });
    }
};

//looks for the currently viewable refresh button and if present presses it.
//Otherwise, it refreshes by itself.
barco.utils.disableCurrentRefreshButton = function() {
    "use strict";
    var refButton = Ext.getCmp("mainpanel").getActiveItem().down('button[text="Refresh"]');
    if(refButton){
        refButton.disable();
    }
};

barco.utils.addInnerPanelToolbar = function(listOfPanelOptions) {
    "use strict";
    var buttonHandler,
        SegButton;
        
    buttonHandler = function(btn, event) {
        btn.ownerCt.changesLocked = true;
        if(btn.ownerCt.allowBothChannels && (barco.evtManager.ChannelBoth != false)) {
            barco.evtManager.fireEvent('ChannelBoth', false);
        }
        btn.ownerCt.lastPressed = btn.buttonIdx;
        barco.evtManager.fireEvent('ChannelChange', btn.buttonIdx);
        btn.ownerCt.changesLocked = false;
    };
    SegButton = Ext.extend(Ext.SegmentedButton, {
        
       //Need explicit override of method afterLayout, adding listener to the config
       //objects does not allow setPressed to be called. 
        initComponent: function() {
            var owner = this;
            if(this.allowBothChannels) {
                this.items.push(new Ext.Button({
                    text: 'Both',
                    buttonIdx: 2,
                    width: 100,
                    hidden: false,
                    allowDepress: true,
                    handler: function(btn, event) {
                        owner.changesLocked = true;
                        barco.evtManager.fireEvent('ChannelBoth', btn.buttonIdx == 2);
                        owner.changesLocked = false;
                    }

                }));
            }
           SegButton.superclass.initComponent.call(this);
        },
        afterLayout: function(event) {
           SegButton.superclass.afterLayout.call(this, event);
           this.switchTo(barco.evtManager.ChannelChange);
        },
        /***********************************************************************
         * SwitchTo
         **********************************************************************/
        switchTo: function(idx) {
            if(this.allowBothChannels && barco.evtManager.ChannelBoth) {
                this.setPressed(2, true);
            } else {
                this.setPressed(idx, true);
            }
        }
    });
    listOfPanelOptions.forEach(function(panelOption) {
        panelOption.allowBothChannels = panelOption.allowBothChannels || false;
        panelOption.panel.on('beforerender', function(container) {
            var tbPanel = new Ext.Panel({
                layout: {
                    type: 'hbox',
                    align: 'end'
                },
                listeners: {
                    show: function(comp) {
                        //this MUST be because of a stupid bug in sencha. 
                        //calling hide in afterrender has no effect sometimes,
                        //so we wait until show event is fired to hide it again.
                        if(barco.evtManager.ShowChannelModeButton === false ) {
                            comp.hide();
                        }
                    },
                    afterrender: function(comp){
                        if(barco.evtManager.ShowChannelModeButton){
                            comp.show();
                        }
                        else{
                            comp.hide();
                        }
                        barco.evtManager.on('ShowChannelModeButton', function(show) {
                            if(show) {
                                comp.show();
                            } else {
                                comp.hide();
                            }
                        });
                    }
                },
                cls: 'channelSelectToolbar-container',
                items: [
                    {
                        xtype: 'spacer'
                    }, {
                        cls: 'channelSelectToolbar',
                        xtype: 'toolbar',
                        ui: 'light',
                        width: panelOption.allowBothChannels ? 314: 214,
                        layout: {
                            pack: 'center'
                        },
                        items: [
                            new SegButton({
                                centered: true,
                                allowMultiple: false,
                                cls: 'brightButtons',
                                allowBothChannels: panelOption.allowBothChannels,
                                listeners: {
                                    afterrender: function(me) {
                                        barco.evtManager.on('ChannelChange', function(idx) {
                                            me.switchTo(idx);
                                        });
                                        if(me.allowBothChannels) {
                                            barco.evtManager.on('ChannelBoth', function(enable) {
                                                if(enable) {
                                                    me.switchTo(2);
                                                } else {
                                                    me.switchTo(barco.evtManager.ChannelChange);
                                                }
                                            });
                                        }
                                    }

                                },
                                defaults: {
                                    width: 100
                                },
                                items: [{
                                    text: 'Channel A',
                                    buttonIdx: 0,
                                    handler: buttonHandler
                                }, {
                                    text: 'Channel B',
                                    buttonIdx: 1,
                                    handler: buttonHandler

                                }]
                            })
                        ]
                    }
            ]
            });

            this.items.items.unshift(tbPanel);
        }, panelOption.panel);
    });
};


window.silencefans = function() {
    "use strict";
    if(window.silencefans.enable !== true) {
        barco.utils.ascii('fanctrl -f 1 -s 30');
        barco.utils.ascii('fanctrl -f 2 -s 50');
        window.silencefans.enable = true;
    } else {
        barco.utils.ascii('fanctrl -f 1 -s 100');
        barco.utils.ascii('fanctrl -f 2 -s 100');
        window.silencefans.enable = false;
    }

};

barco.utils.topLeftToCenter = function(win, HFull, VFull) {
    "use strict";
    var t1, t2;

    t1 = Math.ceil((parseInt(win.HSize, 10) - HFull) / 2);
    t2 = Math.ceil((parseInt(win.VSize, 10) - VFull) / 2);
   
    return [t1+parseInt(win.HPos, 10), -(t2+parseInt(win.Vpos, 10))];
};

barco.utils.selectedInput = function() {
    "use strict";
    return parseInt(barco.utils.xmlselect(barco.tags.ActiveConn)[0].textContent, 10) + 1;
};

barco.utils.dviInputType = function() {
    "use strict";
    return parseInt(barco.utils.xmlselectnode(barco.tags.InType).textContent, 10);
};

barco.utils.isActiveConnDigital = function(){
    "use strict";
	var activeConn = barco.utils.xmlselectnode(barco.tags.ActiveConn).textContent;
	return (activeConn > 1 && activeConn < 6);
};

barco.utils.isDualPixelMode = function(){
    "use strict";
	var pixClk = barco.utils.xmlselectnode(barco.tags.OutputChan + " " + barco.tags.VideoFormat + " " + barco.tags.PixelClk).textContent;
	return barco.utils.isActiveConnDigital() && pixClk > 165.0;
};

barco.utils.isQuadToDualMode = function() {
    "use strict";
    return barco.modelProxy.stores.SystemStore.data.items[0].data.OpMode == 4; 
};

//whether input is something static like LOGO, BLK or FRZ, as opposed to something configurable like DVI or HD15
barco.utils.isInputStatic = function() {
    "use strict";
    var freeze = barco.utils.xmlselectnode(barco.tags.FreezeMode).textContent == 1,
        staticInput = barco.utils.xmlselectnode(barco.tags.ActiveConn).textContent >= 6; //5 is SDI-2, last "dynamic" input
    return freeze || staticInput; 
};

//whether system mode is dual
barco.utils.isDualMode = function() {
    "use strict";
    return barco.modelProxy.stores.SystemStore.data.items[0].data.OpMode == 1; 
};

//whether system mode is standard
barco.utils.isStandardMode = function() {
    "use strict";
    return barco.modelProxy.stores.SystemStore.data.items[0].data.OpMode == 0; 
};

//whether system mode is 3D
barco.utils.is3DMode = function() {
    "use strict";
    return barco.modelProxy.stores.SystemStore.data.items[0].data.OpMode == 2; 
};

//whether system mode is MinDly
barco.utils.isMinDelayMode = function() {
    "use strict";
    return barco.modelProxy.stores.SystemStore.data.items[0].data.OpMode == 3; 
};

barco.utils.isImageProJr = function() {
	"use strict";
	return barco.utils.xmlselectnode("SysType").textContent === "1";
};

barco.utils.getHostname = function() {
    "use strict";
	var title = barco.utils.xmlselectnode(this.Root + " " + barco.tags.Name);
	return title ? title.textContent : barco.settings.title;
};

//check if SDI-2 is installed
barco.utils.isSDI_2Ready = function() {
    "use strict";
    return barco.utils.xmlselectnode("Slot[Num='0'] SlotModuleType").textContent === "2";
};

barco.setResolutionByIndex = function(rawxml, store) {
    "use strict";
    if(store.data.items.length > 0) {
        var inputSel = this, index, outputChanIndex = this.index || 0; 
        index = inputSel.store.find("text", store.data.items[outputChanIndex].data.Name, 0, false, true, true);
        if(index==-1) {
           inputSel.setValue(store.data.items[outputChanIndex].data.Name); //for custom resolutions?
        }
        else {
           inputSel.setValue(index);
        }
    }
};

barco.utils.commandAsciiWithReload = function(command) {
    "use strict";
    var reload = function () {
            setTimeout(function () {
                barco.utils.refreshAll();
            }, 2000);
        };
    barco.utils.disableCurrentRefreshButton();
    //console.debug(command);
    barco.utils.ascii(
        {
            request: command,
            success: reload,
            failure: reload
        }
    );
};

/**
 barco.utils.convertXmlToCmd
    converts a regular XML write to the command format
     Puts: <CmdType>1</CmdType> after the root element
 **/
barco.utils.convertXmlToCmd = function(tpl) {
    "use strict";
    var endOfRoot = tpl.indexOf('>') + 1;
    return tpl.slice(0, endOfRoot) + '<CmdType>1</CmdType>' + tpl.slice(endOfRoot);
};

/*******************************************************************************
 * barco.utils.createWheelDispatcher: dispatcher creates the wheel component and displays it
 *  either by the component or centered when the 'wheelTargetChange' is fired.
 ******************************************************************************/
barco.utils.createWheelDispatcher = function() {
    "use strict";
    var wheel = new barco.Wheel({floating: true, stepSize: 15, width:195, height: 195}),
        cmp = null,
        label = null,
        updater;
    barco.evtManager.addSingleEvent('wheelTargetChange', null);
    barco.evtManager.on('wheelTargetChange', function(obj) {
        label = Ext.get(obj);
        cmp = Ext.getCmp(label.parent().id);
        if (cmp.showWheelPlugin)
        {
            label.addCls('greenhighlight');        
            updater = barco.modelProxy.updateComponentLite(cmp);
            updater.lock();                
            if(Ext.is.Phone) {
                wheel.setCentered(true);                        
                wheel.show();
            } else {
                wheel.showBy(label, false, false);
            }
            wheel.forgetPosition();
        }        
    });
    
    wheel.on('wheelrotate', function(rotation) {
        if(cmp) {
            updater.update(cmp.getValue() + rotation);
        }
    });
    wheel.on('hide', function() {
        label.removeCls('greenhighlight');
        updater.finish();
    });
    Ext.Viewport.on('resize', function(){
       wheel.hide(); 
    });
};

/*******************************************************************************
 *  barco.utils.asteriskIcon - a utility for adding the asterisk icon to form labels
 *      returns object containing methods: add, remove
 *         barco.utils.asteriskIcon().add(cmp)
 *         barco.utils.asteriskIcon().remove(cmp)
 ******************************************************************************/
barco.utils.asteriskIcon = function() {
    "use strict";
    var asObj = {},
        imageLocation = '"/images/asterisk.png"',
        img = '<span class="asteriskIcon"><img src=' + imageLocation + '>' +
                 '<div class="tooltip">Channel A/B values vary. Showing A</div>' +
              '</span>' ,
        selector = 'span.asteriskIcon';
        
    function privateGetDom(component) {
        var dom;
        if(component.el !== undefined) {
            dom = component.el.dom;
        } else {
            dom = component.dom;
        }
        return dom;
    }
    asObj.add = function(component) {
        var label = Ext.get(Ext.DomQuery.selectNode('.x-form-label', privateGetDom(component)));
        if(label.child(selector) === null) {
            //create child img and prevent user from dragging it
            label.createChild(img).on('touchstart', function(evt) {evt.preventDefault();});
        }
    };
    asObj.remove = function(component) {
        var label = Ext.get(Ext.DomQuery.selectNode('.x-form-label', privateGetDom(component))),
            child = label.child(selector);
        if(child !== null) {
            child.remove();
        }
    };
    return asObj;
};

//returns a subset of *array* given by *indices* which is an array of indices
barco.utils.filterByIndices = function(array, indices){
    "use strict";
    var newarr = [], x;
    for(x=0; x<indices.length; x+=1){
        newarr.push(array[indices[x]]);
    }
    return newarr;
};
/**
 *Return a list of all the leafs for a given XML node
 **/
barco.utils.findAllXmlNodeLeafs = function(node) {
    "use strict";
    var getNonTextChildCount,
        findLeafs;
        
    getNonTextChildCount = function(node) {
        var i,
            count = 0;
        for(i = 0; i < node.childNodes.length; i+=1) {
            if(node.childNodes[i].nodeType !== 3) {
                count += 1;
            }
        }
        return count;
    };
    
    
    findLeafs = function(elem, path, allMyChildren) {
        var idx,
            node,
            mypath = path + " " + elem.tagName;
            
        //add all Node attributes
        Array.prototype.forEach.call(elem.attributes, function(attr) {
            mypath = mypath.concat('[' + attr.name + '=' + '"' + attr.value + '"' + ']');
        });
        for(idx = 0; idx < elem.childNodes.length; idx+=1) {
            node = elem.childNodes[idx];
            if(node.nodeType != Node.TEXT_NODE)/*avoid text node*/ {
                
                //console.log('elem = ' + elem.tagName + '\n\t path =' + path + '\n child = '  + node.tagName);
                if(getNonTextChildCount(node) == 0) {
                    //is a leaf
                    //console.log('got a leaf -- ' + node.tagName);
                    allMyChildren.push({node: node, path: mypath + ' ' + node.tagName});
                } else {
                    allMyChildren = findLeafs(node, mypath, allMyChildren);
                }
            }
        }
        return allMyChildren;
    };
    
    return findLeafs(node, '', []);
};