Ext.ns('barco');

barco.Wheel = Ext.extend(Ext.Component, {
    cls: 'vbox boxcenter',
    html: '<div class="hbox boxcenter">' + 
                '<div class="vbox boxcenter">' +
                        '<img class="b-wheel" src="/../images/wheel.png"/>' + 
                '</div>' +
            '</div>',
    id: 'testpanel',
    radius: 87.5,
    stepSize: 10,
    enableThrottle: true,
    constructor: function(config) {
        "use strict";
        barco.Wheel.superclass.constructor.call(this, config);
        this.addEvents('wheelrotate');
    },
    forgetPosition: function() {
        "use strict";
        this.wheel.offsetLast = {};
    },
    rotateToPoint: function(x, y) {
        "use strict";
        var deg;
        x = x - this.radius;
        y = y - this.radius;
        deg = Math.atan(y/x) * (90.0/(Math.PI/2));
        if(y > 0) {
//            if(x > 0) {
//                //quad = 1;
//            }
            if(x<0) {
                deg = 180+deg;
                //quad = 2;
            }
        } else {
            if(x > 0) {
                //quad = 4;
                deg = 360+deg;
            } else {
                deg = 180+deg;
                //quad = 3;
            }
        }
        //make up for wheel facing upwards
        deg = deg + 90;
        
        this.rotateToAngle(deg);
    },
    rotateToAngle: function(degrees) {
        "use strict";
        var modChange = degrees % 360,
            currQuad,
            diffAngle,
            throttle,
            me = this;
        if(modChange < this.wheel.angleOffset) {
            this.wheel.angleOffset = this.wheel.angleOffset + ((360-(this.wheel.angleOffset%360)) + modChange);
        } else {
            this.wheel.angleOffset = modChange;
        }
        function getQuadrant(angle) {
            if( (0 < angle) && (angle <= 90)) {
                return 1;
            } else if( (90 < angle) && (angle <= 180)) {
                return 2;
            } else if( (180 < angle) && (angle <= 270)) {
                return 3;
            } else if( (270 < angle) && (angle <= 360)) {
                return 4;
            }
            return -1;
        }
        
        currQuad = getQuadrant(modChange);
        
        //console.log('modChange ' + modChange + ' quad = ' + currQuad + ' last quad = ' + this.wheel.lastQuadrant);
        
        diffAngle = (modChange - this.wheel.lastModChange);
        
        
        //prevent roll over
        if(this.wheel.lastQuadrant === 4 || (currQuad === 1 && this.wheel.lastQuadrant === 3)) {
            //make adjustment for going over 360
            diffAngle = (360-this.wheel.lastModChange) + modChange;
        }
        
        //change can be considered positive or negative depending on half sector. 
        if( (diffAngle > 180))  {
            diffAngle =  diffAngle - 360;
        } else if(diffAngle < -180) {
            diffAngle = diffAngle + 180;
        }
        
        if(Math.abs(diffAngle) > this.stepSize) { 
                
            this.wheel.lastQuadrant = currQuad;
            this.wheel.lastModChange = modChange;
            if(this.enableThrottle) {
                throttle = function() {
                    me.throttleTimer = null;
                    me.fireEvent('wheelrotate', me.totalThrottled);
                    
                };
                
                if(!this.throttleTimer) {
                    this.totalThrottled = 0;
                    this.throttleTimer = setTimeout(throttle, 100);
                }
                this.totalThrottled = this.totalThrottled + parseInt(diffAngle/(this.stepSize), 10);
            }
            else {
                this.fireEvent('wheelrotate', parseInt(diffAngle/(this.stepSize), 10));
            }
        }
        this.wheel.dom.style.cssText = "-webkit-transform:rotate(" + degrees + "deg);";
        
    },
    afterRender: function() {
        "use strict";
        
        var onDown = function(evt, elem) {
                evt.preventDefault();
                var offsetX, offsetY, event, divPosition;
                if(evt.browserEvent !== undefined) {
                    event = evt.browserEvent;
                } else {
                    event = evt.event;
                }
                divPosition = this.wheel.getAbsPosition();
                if(Ext.is.Desktop === false) {    
                    offsetX = evt.targetTouches[0].pageX - divPosition.currleft;
                    offsetY = evt.targetTouches[0].pageY - divPosition.currtop;
                } else {
                    offsetX = event.pageX - divPosition.currleft;
                    offsetY = event.pageY - divPosition.currtop;
                }
                
                this.rotateToPoint(offsetX, offsetY);
        };
        this.wheel = Ext.get(Ext.DomQuery.selectNode('img', this.el.dom));
        this.wheel.angleOffset = 0;
        this.wheel.offsetLast = {};
        this.wheel.lastQuadrant = 1;
        this.wheel.lastModChange = 0;
        /* Get coordinates of the wheel*/
        this.wheel.getAbsPosition = function() {
            var currleft = 0,
                currtop = 0,
                obj = this.dom;
            if(obj.offsetLeft !== this.offsetLast.offsetLeft || obj.offsetTop !== this.offsetLast.offsetTop) {
                //position has changed
                this.offsetLast.offsetLeft = obj.offsetLeft;
                this.offsetLast.offsetTop = obj.offsetTop;
                if(obj.offsetParent) {
                    do {
                        currleft += obj.offsetLeft;
                        currtop += obj.offsetTop;
                        obj = obj.offsetParent;
                    } while( obj!== null);
                }
                //cache
                this.offsetLast.currleft = currleft;
                this.offsetLast.currtop = currtop;
            } else {
                //use cache
                currleft = this.offsetLast.currleft;
                currtop = this.offsetLast.currtop;
            }
            
            //console.log("getWheelAbsPosition: " + currleft + " " + currtop);
            return {'currleft': currleft, 'currtop': currtop};
        };
        
        this.wheel.on({
            touchstart: onDown,
            drag: function(evt, elem) {
                if(elem === this.wheel.dom) {
                    var offsetX, offsetY, divPosition;
                    divPosition = this.wheel.getAbsPosition();
                    if(Ext.is.Desktop) {
                        offsetX = evt.event.pageX - divPosition.currleft;
                        offsetY = evt.event.pageY - divPosition.currtop;
                    } else {
                        offsetX = evt.targetTouches[0].pageX - divPosition.currleft;
                        offsetY = evt.targetTouches[0].pageY - divPosition.currtop;
                    }
                    this.rotateToPoint(offsetX, offsetY);
                }
            },
            scope: this
        });
    }
});
